MODULE p4zdiat
   !!======================================================================
   !!                         ***  MODULE p4zdiat  ***
   !! TOP :   PISCES Compute the mortality terms for diatoms
   !!======================================================================
   !! History :   1.0  !  2002     (O. Aumont)  Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces
   !!----------------------------------------------------------------------
   !!   'key_pisces'                                       PISCES bio-model
   !!----------------------------------------------------------------------
   !!   p4z_diat       :   Compute the mortality terms for diatoms
   !!----------------------------------------------------------------------
   USE oce_trc         !
   USE trp_trc         ! 
   USE sms             ! 

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_diat    ! called in p4zbio.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_diat
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_diat  ***
      !!
      !! ** Purpose :   Compute the mortality terms for diatoms
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER  ::   ji, jj, jk
      REAL(wp) ::   zfact, zstep, zcompadi
      !!---------------------------------------------------------------------

        zstep = rfact2 / rjjss      ! Time step duration for biology


!    Aggregation term for diatoms is increased in case of nutrient
!    stress as observed in reality. The stressed cells become more
!    sticky and coagulate to sink quickly out of the euphotic zone
!     ------------------------------------------------------------

      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi

               zcompadi = MAX( ( trn(ji,jj,jk,jpdia) - 1e-8), 0. )
               zfact    = 1. / ( trn(ji,jj,jk,jpdia) + rtrn )

!    Aggregation term for diatoms is increased in case of nutrient
!    stress as observed in reality. The stressed cells become more
!    sticky and coagulate to sink quickly out of the euphotic zone
!     ------------------------------------------------------------

               respp2 (ji,jj,jk) = 1.e6 * zstep * (  wchl + wchld * ( 1.- xlimdia(ji,jj,jk) )  )    &
# if defined key_off_degrad
                  &              * facvol(ji,jj,jk)       &
# endif
                  &              * zdiss(ji,jj,jk) * zcompadi * trn(ji,jj,jk,jpdia)
                                                                               
               respds (ji,jj,jk) = respp2(ji,jj,jk) * trn(ji,jj,jk,jpbsi) * zfact

               respdf (ji,jj,jk) = respp2(ji,jj,jk) * trn(ji,jj,jk,jpdfe) * zfact
                                                                               
               respdch(ji,jj,jk) = respp2(ji,jj,jk) * trn(ji,jj,jk,jpdch) * zfact

!     Phytoplankton mortality. 
!     ------------------------
               tortp2 (ji,jj,jk) = mprat2 * zstep * trn(ji,jj,jk,jpdia)     &
# if defined key_off_degrad
                  &              * facvol(ji,jj,jk)       &
# endif
                  &              / ( xkmort + trn(ji,jj,jk,jpdia) ) * zcompadi

               tortds (ji,jj,jk) = tortp2(ji,jj,jk) * trn(ji,jj,jk,jpbsi) * zfact

               tortdf (ji,jj,jk) = tortp2(ji,jj,jk) * trn(ji,jj,jk,jpdfe) * zfact

               tortdch(ji,jj,jk) = tortp2(ji,jj,jk) * trn(ji,jj,jk,jpdch) * zfact

            END DO
         END DO
      END DO
      !
   END SUBROUTINE p4z_diat

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_diat                    ! Empty routine
   END SUBROUTINE p4z_diat
#endif 

   !!======================================================================
END MODULE  p4zdiat
