MODULE p4zint
   !!======================================================================
   !!                         ***  MODULE p4zint  ***
   !! TOP :   PISCES interpolation and computation of various accessory fields
   !!======================================================================
   !! History :   1.0  !  2004-03 (O. Aumont) Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces
   !!----------------------------------------------------------------------
   !!   'key_pisces'                                       PISCES bio-model
   !!----------------------------------------------------------------------
   !!   p4z_int        :  interpolation and computation of various accessory fields
   !!----------------------------------------------------------------------
   USE oce_trc         !
   USE trp_trc
   USE sms

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_int    ! called in p4zprg.F90

   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_int( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_int  ***
      !!
      !! ** Purpose :   interpolation and computation of various accessory fields
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER, INTENT( in ) ::   kt      ! ocean time-step index      
      !!
      INTEGER  ::   ji, jj
      INTEGER  ::   iman, ivit1t, ivit2t
      REAL(wp) ::   zpdtan, zman, zpdtmo, zdemi
      REAL(wp) ::   zt, zdum
      !!---------------------------------------------------------------------

      zpdtan = raass / rdt
      zman   = 12.
      iman   = 12
      zpdtmo = zpdtan / zman
      zdemi  = zpdtmo / 2.
      zt     = ( float ( kt) + zdemi) / zpdtmo
      
      !  recherche de l indice des enregistrements
      !  du modele dynamique encadrant le pas de temps kt.
      !  --------------------------------------------------

      xtvit = zt - FLOAT( INT( zt ) )
      ivit1t = INT( zt )
      ivit2t = ivit1t + 1
      ivit1t = MOD ( ivit1t, iman )
      IF( ivit1t == 0 )   ivit1t = iman
      ivit2t = MOD ( ivit2t, iman )
      IF( ivit2t == 0 )   ivit2t = iman

      ! Interpolation of dust deposition
      ! --------------------------------

      dust(:,:) = ( 1.- xtvit ) * dustmo(:,:,ivit1t) + xtvit * dustmo(:,:,ivit2t)

      ! Computation of phyto and zoo metabolic rate
      ! -------------------------------------------

      Tgfunc (:,:,:) = EXP( 0.063913 * tn(:,:,:) )
      Tgfunc2(:,:,:) = EXP( 0.07608  * tn(:,:,:) )

      ! Computation of the silicon dependant half saturation
      ! constant for silica uptake
      ! ---------------------------------------------------

      DO ji = 1, jpi
         DO jj = 1, jpj
            zdum = trn(ji,jj,1,jpsil) * trn(ji,jj,1,jpsil)
            xksimax(ji,jj) = MAX( xksimax(ji,jj), ( 1.+ 7.* zdum / ( xksi2 * xksi2 * 25. + zdum ) ) * 1e-6 )
         END DO
      END DO

      IF( nday_year == 365 ) THEN
         xksi    = xksimax
         xksimax = 0.e0
      ENDIF
      !
   END SUBROUTINE p4z_int

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_int( kt )                   ! Empty routine
      INTEGER, INTENT( in ) ::   kt
      WRITE(*,*) 'p4z_int: You should not have seen this print! error?', kt
   END SUBROUTINE p4z_int
#endif 

   !!======================================================================
END MODULE  p4zint
