MODULE p4znano
   !!======================================================================
   !!                         ***  MODULE p4znano  ***
   !! TOP :   PISCES Compute the mortality terms for nanophytoplankton
   !!======================================================================
   !! History :   1.0  !  2002     (O. Aumont) Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces
   !!----------------------------------------------------------------------
   !!   'key_pisces'                                       PISCES bio-model
   !!----------------------------------------------------------------------
   !!   p4z_nano       :   Compute the mortality terms for nanophytoplankton
   !!----------------------------------------------------------------------
   USE oce_trc         !
   USE trp_trc         ! 
   USE sms             ! 

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_nano    ! called in p4zbio.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_nano
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_nano  ***
      !!
      !! ** Purpose :   Compute the mortality terms for nanophytoplankton
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER  ::   ji, jj, jk
      REAL(wp) ::   zfact, zstep, zcompaph
      !!---------------------------------------------------------------------


      zstep = rfact2 / rjjss      ! Time step duration for biology


      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi

               zcompaph = MAX( ( trn(ji,jj,jk,jpphy) - 1e-8 ), 0.e0 )
               zfact    = 1./ ( trn(ji,jj,jk,jpphy) + rtrn )

!     Squared mortality of Phyto similar to a sedimentation term during
!     blooms (Doney et al. 1996)
!     -----------------------------------------------------------------
               respp(ji,jj,jk) = wchl * 1.e6 * zstep * zdiss(ji,jj,jk)   &
# if defined key_off_degrad
                  &            * facvol(ji,jj,jk)     &
# endif
                  &            * zcompaph * trn(ji,jj,jk,jpphy)
                                                                               
               respnf (ji,jj,jk) = respp(ji,jj,jk) * trn(ji,jj,jk,jpnfe) * zfact
                                                                               
               respnch(ji,jj,jk) = respp(ji,jj,jk) * trn(ji,jj,jk,jpnch) * zfact

!     Phytoplankton mortality. This mortality loss is slightly
!     increased when nutrients are limiting phytoplankton growth
!     as observed for instance in case of iron limitation.
!     ----------------------------------------------------------
               tortp  (ji,jj,jk) = mprat * zstep * trn(ji,jj,jk,jpphy)          &
# if defined key_off_degrad
                  &              * facvol(ji,jj,jk)     &
# endif
                  &              / ( xkmort + trn(ji,jj,jk,jpphy) ) * zcompaph
                                                                               
               tortnf (ji,jj,jk) = tortp(ji,jj,jk) * trn(ji,jj,jk,jpnfe) * zfact
                                                                               
               tortnch(ji,jj,jk) = tortp(ji,jj,jk) * trn(ji,jj,jk,jpnch) * zfact

            END DO
         END DO
      END DO
      !
   END SUBROUTINE p4z_nano

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_nano                    ! Empty routine
   END SUBROUTINE p4z_nano
#endif 

   !!======================================================================
END MODULE  p4znano
