MODULE p4zopt
   !!======================================================================
   !!                         ***  MODULE p4zopt  ***
   !! TOP :   PISCES Compute the light availability in the water column
   !!======================================================================
   !! History :   1.0  !  2004     (O. Aumont) Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces
   !!----------------------------------------------------------------------
   !!   'key_pisces'                                       PISCES bio-model
   !!----------------------------------------------------------------------
   !!   p4z_opt        :   Compute the light availability in the water column
   !!----------------------------------------------------------------------
   USE oce_trc         !
   USE trp_trc
   USE sms

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_opt   ! called in p4zprg.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_opt
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE p4z_opt  ***
      !!
      !! ** Purpose :   Compute the light availability in the water column
      !!              depending on the depth and the chlorophyll concentration
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER  ::   ji, jj, jk
      INTEGER  ::   irgb
      REAL(wp) ::   zchl, zparlux
      REAL(wp) ::   zrlight , zblight , zglight
      REAL(wp) ::   zrlight1, zblight1, zglight1
      REAL(wp), DIMENSION(jpi,jpj)     ::   zdepmoy, zetmp
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zekg, zekr, zekb
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   ze1 , ze2 , ze3
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   ze3lum, ze4lum
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   ze5lum, ze6lum
      !!---------------------------------------------------------------------

!     Initialisation of variables used to compute PAR
!     -----------------------------------------------
      ze1 (:,:,:) = 0.e0
      ze2 (:,:,:) = 0.e0
      ze3 (:,:,:) = 0.e0
      etot(:,:,:) = 0.e0
        
      zparlux = 0.43 / 3.

!    IF activated, computation of the qsr for the dynamics
!    -----------------------------------------------------
      IF( ln_qsr_sms ) THEN
         ze3lum(:,:,:) = 0.e0
         ze4lum(:,:,:) = 0.e0
         ze5lum(:,:,:) = 0.e0
         ze6lum(:,:,:) = 0.e0
      ENDIF

      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi

!     Separation in three light bands: red, green, blue
!     -------------------------------------------------
               zchl = ( trn(ji,jj,jk,jpnch) + trn(ji,jj,jk,jpdch) + rtrn ) * 1.e6
               zchl = MAX( 0.03, zchl )
               zchl = MIN( 10. , zchl )
                                                                                
               irgb = INT( 41 + 20.* LOG10( zchl ) + rtrn )
                                                                                
               zekb(ji,jj,jk) = xkrgb(1,irgb)
               zekg(ji,jj,jk) = xkrgb(2,irgb)
               zekr(ji,jj,jk) = xkrgb(3,irgb)

            END DO
         END DO
      END DO

      DO jj = 1,jpj
         DO ji = 1,jpi

!     Separation in three light bands: red, green, blue
!     -------------------------------------------------

            zblight = 0.5 * zekb(ji,jj,1) * fse3t(ji,jj,1)
            zglight = 0.5 * zekg(ji,jj,1) * fse3t(ji,jj,1)
            zrlight = 0.5 * zekr(ji,jj,1) * fse3t(ji,jj,1)

            ze1(ji,jj,1) = zparlux * qsr(ji,jj) * EXP(-zblight)
            ze2(ji,jj,1) = zparlux * qsr(ji,jj) * EXP(-zglight)
            ze3(ji,jj,1) = zparlux * qsr(ji,jj) * EXP(-zrlight)

         END DO
      END DO

      DO jk = 2, jpkm1
          DO jj = 1, jpj
            DO ji = 1, jpi

!     Separation in three light bands: red, green, blue
!     -------------------------------------------------

               zblight = 0.5 * ( zekb(ji,jj,jk-1) * fse3t(ji,jj,jk-1)   &
                  &            + zekb(ji,jj,jk  ) * fse3t(ji,jj,jk  ) )
               zglight = 0.5 * ( zekg(ji,jj,jk-1) * fse3t(ji,jj,jk-1)   &
                  &            + zekg(ji,jj,jk  ) * fse3t(ji,jj,jk  ) )
               zrlight = 0.5 * ( zekr(ji,jj,jk-1) * fse3t(ji,jj,jk-1)   &
                  &            + zekr(ji,jj,jk  ) * fse3t(ji,jj,jk  ) )

               ze1(ji,jj,jk) = ze1(ji,jj,jk-1) * EXP(-zblight)
               ze2(ji,jj,jk) = ze2(ji,jj,jk-1) * EXP(-zglight)
               ze3(ji,jj,jk) = ze3(ji,jj,jk-1) * EXP(-zrlight)

            END DO
         END DO
      END DO

      etot(:,:,:) = ze1(:,:,:) + ze2(:,:,:) + ze3(:,:,:)

      IF( ln_qsr_sms ) THEN

!   In the following, the vertical attenuation of qsr for the dynamics is computed
!   ------------------------------------------------------------------------------

         DO jj = 1, jpj
            DO ji = 1, jpi

!     Separation in three light bands: red, green, blue
!     -------------------------------------------------

               zblight = 0.5 * zekb(ji,jj,1) * fse3t(ji,jj,1)
               zglight = 0.5 * zekg(ji,jj,1) * fse3t(ji,jj,1)
               zrlight = 0.5 * zekr(ji,jj,1) * fse3t(ji,jj,1)

               ze3lum(ji,jj,1) = zparlux * qsr(ji,jj)
               ze4lum(ji,jj,1) = zparlux * qsr(ji,jj)
               ze5lum(ji,jj,1) = zparlux * qsr(ji,jj)
               ze6lum(ji,jj,1) = (1.-3. * zparlux) * qsr(ji,jj)

            END DO
         END DO

         DO jk = 2, jpkm1
            DO jj = 1, jpj
               DO ji = 1, jpi

!     Separation in three light bands: red, green, blue
!     -------------------------------------------------

                  zblight1 = zekb(ji,jj,jk-1) * fse3t(ji,jj,jk-1)
                  zglight1 = zekg(ji,jj,jk-1) * fse3t(ji,jj,jk-1)
                  zrlight1 = zekr(ji,jj,jk-1) * fse3t(ji,jj,jk-1)

                  ze3lum(ji,jj,jk) = ze3lum(ji,jj,jk-1) * EXP( -zblight )
                  ze4lum(ji,jj,jk) = ze4lum(ji,jj,jk-1) * EXP( -zglight )
                  ze5lum(ji,jj,jk) = ze5lum(ji,jj,jk-1) * EXP( -zrlight )
                  ze6lum(ji,jj,jk) = ze6lum(ji,jj,jk-1) * EXP( -fse3t(ji,jj,jk-1) / xsi1 )

               END DO
            END DO
         END DO

         etot3(:,:,:) = ze3lum(:,:,:) + ze4lum(:,:,:) + ze5lum(:,:,:) + ze6lum(:,:,:)

      ENDIF

!     Computation of the euphotic depth
!     ---------------------------------
    
      zmeu(:,:) = 300.e0

      DO jk = 2, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( etot(ji,jj,jk) >= 0.0043 * qsr(ji,jj) )   zmeu(ji,jj) = fsdepw(ji,jj,jk+1)
            END DO
         END DO
      END DO

      zmeu(:,:) = MIN( 300., zmeu(:,:) )

!    Computation of the mean light over the mixed layer depth
!    --------------------------------------------------------

      zdepmoy(:,:)   = 0.e0
      zetmp  (:,:)   = 0.e0
      emoy   (:,:,:) = 0.e0

      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( fsdepw(ji,jj,jk+1) <= hmld(ji,jj) ) THEN
                  zetmp  (ji,jj) = zetmp  (ji,jj) + etot(ji,jj,jk) * fse3t(ji,jj,jk)
                  zdepmoy(ji,jj) = zdepmoy(ji,jj) +                  fse3t(ji,jj,jk)
               ENDIF
            END DO
         END DO
      END DO

      emoy(:,:,:) = etot(:,:,:)

      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( fsdepw(ji,jj,jk+1) <= hmld(ji,jj) ) THEN
                  emoy(ji,jj,jk) = zetmp(ji,jj) / ( zdepmoy(ji,jj) + rtrn )
               ENDIF
            END DO
         END DO
      END DO

# if defined key_trc_diaadd
      trc2d(:,:,11) = zmeu(:,:)
# endif
      !
   END SUBROUTINE p4z_opt

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_opt                   ! Empty routine
   END SUBROUTINE p4z_opt
#endif 

   !!======================================================================
END MODULE  p4zopt
